@@ -10,14 +10,11 @@ import typing as tp
 import jax.numpy as jnp
 import numpy as np
 import six
-
-from elegy import types
-from elegy.types import ArrayHolder
-from elegy import utils
+from elegy import types, utils
 
 
-class Multimap(utils.Protocol):
-    def __call__(self, *args: types.ArrayLike) -> types.T:
+class Multimap(types.Protocol):
+    def __call__(self, *args: types.np.ndarray) -> types.T:
         ...
 
 
@@ -27,7 +24,8 @@ def map_append(output, batch_output):
 
 
 def map_structure(
-    f: Multimap, *structure: tp.Union[types.ArrayHolder, None],
+    f: Multimap,
+    *structure: tp.Union[types.ArrayHolder, None],
 ) -> types.Container[tp.Union[types.T, None]]:
 
     if isinstance(structure[0], tp.Tuple):
@@ -43,7 +41,7 @@ def map_structure(
         return None
 
 
-def flatten(inputs: types.ArrayHolder) -> tp.Iterable[types.ArrayLike]:
+def flatten(inputs: types.ArrayHolder) -> tp.Iterable[types.np.ndarray]:
 
     if isinstance(inputs, (jnp.ndarray, np.ndarray)):
         yield inputs
@@ -130,7 +128,7 @@ def assert_not_namedtuple(x):
         )
 
 
-def train_validation_split(arrays, validation_split, shuffle=True):
+def train_validation_split(arrays, validation_split, shuffle=True) -> tp.Tuple:
     """
     Split arrays into random train and validation subsets.
     Arguments:
@@ -144,7 +142,7 @@ def train_validation_split(arrays, validation_split, shuffle=True):
             become the validation split.
     Returns:
         `(train_arrays, validation_arrays)`
-  """
+    """
 
     def _can_split(t):
         supported_types = (jnp.ndarray, np.ndarray)
