@@ -10,7 +10,7 @@ from elegy.metrics.mean import Mean
 class MeanSquaredError(Mean):
     """
     Computes the cumulative mean squared error between `y_true` and `y_pred`.
-    
+
     Usage:
     ```python
     mse = elegy.metrics.MeanSquaredError()
@@ -23,28 +23,21 @@ class MeanSquaredError(Mean):
     ```
 
     Usage with elegy API:
-    
+
     ```python
     model = elegy.Model(
         module_fn,
         loss=elegy.losses.CategoricalCrossentropy(),
-        metrics=elegy.metrics.MeanSquaredError.defer(),
+        metrics=elegy.metrics.MeanSquaredError(),
     )
     ```
     """
 
-    def __init__(
-        self,
-        name: tp.Optional[str] = None,
-        dtype: tp.Optional[jnp.dtype] = None,
-        on: tp.Optional[types.IndexLike] = None,
-    ):
+    def __init__(self, on: tp.Optional[types.IndexLike] = None, **kwargs):
         """
         Creates a `MeanSquaredError` instance.
 
         Arguments:
-            name: string name of the metric instance.
-            dtype: data type of the metric result.
             on: A string or integer, or iterable of string or integers, that
                 indicate how to index/filter the `y_true` and `y_pred`
                 arguments before passing them to `call`. For example if `on = "a"` then
@@ -52,8 +45,9 @@ class MeanSquaredError(Mean):
                 the structures will be indexed iteratively, for example if `on = ["a", 0, "b"]`
                 then `y_true = y_true["a"][0]["b"]`, same for `y_pred`. For more information
                 check out [Keras-like behavior](https://poets-ai.github.io/elegy/guides/modules-losses-metrics/#keras-like-behavior).
+            kwargs: Additional keyword arguments passed to Module.
         """
-        super().__init__(name=name, dtype=dtype, on=on)
+        super().__init__(on=on, **kwargs)
 
     def call(
         self,
@@ -63,7 +57,7 @@ class MeanSquaredError(Mean):
     ) -> jnp.ndarray:
         """
         Accumulates metric statistics. `y_true` and `y_pred` should have the same shape.
-        
+
         Arguments:
             y_true: Ground truth values. shape = `[batch_size, d0, .. dN]`.
             y_pred: The predicted values. shape = `[batch_size, d0, .. dN]`.
@@ -78,6 +72,6 @@ class MeanSquaredError(Mean):
                 functions reduce by 1 dimension, usually the last axis (-1)).
         Returns:
             Array with the cumulative accuracy.
-    """
+        """
 
         return super().call(values=mean_squared_error(y_true=y_true, y_pred=y_pred))
