@@ -20,7 +20,7 @@ def sparse_categorical_accuracy(
 class SparseCategoricalAccuracy(Mean):
     """
     Calculates how often predictions matches integer labels.
-    
+
     You can provide logits of classes as `y_pred`, since argmax of
     logits and probabilities are same.
     This metric creates two local variables, `total` and `count` that are used to
@@ -29,13 +29,13 @@ class SparseCategoricalAccuracy(Mean):
     that simply divides `total` by `count`.
     If `sample_weight` is `None`, weights default to 1.
     Use `sample_weight` of 0 to mask values.
-    
+
     Usage:
     ```python
     accuracy = elegy.metrics.SparseCategoricalAccuracy()
 
     result = accuracy(
-        y_true=jnp.array([2, 1]), 
+        y_true=jnp.array([2, 1]),
         y_pred=jnp.array([[0.1, 0.9, 0.8], [0.05, 0.95, 0]])
     )
     assert result == 0.5  # 1/2
@@ -46,31 +46,24 @@ class SparseCategoricalAccuracy(Mean):
     )
     assert result == 0.75  # 3/4
     ```
-    
+
     Usage with elegy API:
 
     ```python
     model = elegy.Model(
         module_fn,
         loss=elegy.losses.CategoricalCrossentropy(),
-        metrics=elegy.metrics.SparseCategoricalAccuracy.defer(),
-        optimizer=optix.adam(1e-3),
+        metrics=elegy.metrics.SparseCategoricalAccuracy(),
+        optimizer=optax.adam(1e-3),
     )
     ```
     """
 
-    def __init__(
-        self,
-        name: tp.Optional[str] = None,
-        dtype: tp.Optional[jnp.dtype] = None,
-        on: tp.Optional[types.IndexLike] = None,
-    ):
+    def __init__(self, on: tp.Optional[types.IndexLike] = None, **kwargs):
         """
         Creates a `SparseCategoricalAccuracy` instance.
 
         Arguments:
-            name: string name of the metric instance.
-            dtype: data type of the metric result.
             on: A string or integer, or iterable of string or integers, that
                 indicate how to index/filter the `y_true` and `y_pred`
                 arguments before passing them to `call`. For example if `on = "a"` then
@@ -78,8 +71,9 @@ class SparseCategoricalAccuracy(Mean):
                 the structures will be indexed iteratively, for example if `on = ["a", 0, "b"]`
                 then `y_true = y_true["a"][0]["b"]`, same for `y_pred`. For more information
                 check out [Keras-like behavior](https://poets-ai.github.io/elegy/guides/modules-losses-metrics/#keras-like-behavior).
+            kwargs: Additional keyword arguments passed to Module.
         """
-        super().__init__(name=name, dtype=dtype, on=on)
+        super().__init__(on=on, **kwargs)
 
     def call(
         self,
@@ -90,7 +84,7 @@ class SparseCategoricalAccuracy(Mean):
         """
         Accumulates metric statistics. `y_true` and `y_pred` should have the same shape except
         `y_true` should not have the last dimension of `y_pred`.
-        
+
         Arguments:
             y_true: Sparse ground truth values. shape = `[batch_size, d0, .. dN-1]`.
             y_pred: The predicted values. shape = `[batch_size, d0, .. dN-1, dN]`.
@@ -105,7 +99,7 @@ class SparseCategoricalAccuracy(Mean):
                 functions reduce by 1 dimension, usually the last axis (-1)).
         Returns:
             Array with the cumulative accuracy.
-    """
+        """
 
         return super().call(
             values=sparse_categorical_accuracy(y_true=y_true, y_pred=y_pred),
