@@ -1,6 +1,6 @@
 import elegy
-import haiku as hk
-from haiku.testing import transform_and_run
+
+
 import jax.numpy as jnp
 
 
@@ -11,17 +11,18 @@ import jax.numpy as jnp
 # debugpy.wait_for_client()
 
 
-@transform_and_run
+#
 def test_basic():
 
     accuracy = elegy.metrics.SparseCategoricalAccuracy()
 
-    result = accuracy(
+    result = accuracy.call_with_defaults()(
         y_true=jnp.array([2, 1]), y_pred=jnp.array([[0.1, 0.9, 0.8], [0.05, 0.95, 0]])
     )
     assert result == 0.5  # 1/2
 
-    result = accuracy(
-        y_true=jnp.array([1, 1]), y_pred=jnp.array([[0.1, 0.9, 0.8], [0.05, 0.95, 0]]),
+    result = accuracy.call_with_defaults()(
+        y_true=jnp.array([1, 1]),
+        y_pred=jnp.array([[0.1, 0.9, 0.8], [0.05, 0.95, 0]]),
     )
     assert result == 0.75  # 3/4
