@@ -13,6 +13,7 @@ import jax
 import jax.numpy as jnp
 import numpy as np
 import optax
+import toolz
 import yaml
 from elegy import hooks, module, types, utils
 from elegy.losses.loss import Loss
@@ -23,19 +24,6 @@ from tabulate import tabulate
 class PredStep(tp.NamedTuple):
     y_pred: tp.Any
     states: types.States
-    aux_losses: types.Logs
-    aux_metrics: types.Logs
-    summaries: types.Summaries
-
-    @classmethod
-    def simple(cls, y_pred: tp.Any, states: types.States):
-        return cls(
-            y_pred=y_pred,
-            states=states,
-            aux_losses=hooks.get_losses(),
-            aux_metrics=hooks.get_metrics(),
-            summaries=hooks.get_summaries(),
-        )
 
 
 class TestStep(tp.NamedTuple):
@@ -61,41 +49,46 @@ class ModelCore:
     initial_states: types.States
     history: tp.Dict[str, tp.Any]
     run_eagerly: bool = False
+    init_stage: types.Mode = types.Mode.none
 
     def __init__(
         self,
-        net_params: tp.Union[types.Uninitialized, tp.Any] = types.UNINITIALIZED,
-        net_states: tp.Union[types.Uninitialized, tp.Any] = types.UNINITIALIZED,
-        metrics_states: tp.Union[types.Uninitialized, tp.Any] = types.UNINITIALIZED,
-        optimizer_states: tp.Union[types.Uninitialized, tp.Any] = types.UNINITIALIZED,
-        rng: tp.Union[types.Uninitialized, tp.Any] = types.UNINITIALIZED,
+        states: tp.Optional[types.States] = None,
         run_eagerly: bool = False,
+        init_stage: types.Mode = types.Mode.none,
     ):
-        self.states = types.States(
-            net_params=net_params,
-            net_states=net_states,
-            metrics_states=metrics_states,
-            optimizer_states=optimizer_states,
-            rng=rng,
-        )
-        self.initial_states = self.states.copy()
+
+        base_states = self.base_states()
+
+        if states is None:
+            states = types.States()
+
+        states = states.maybe_update(**base_states)
+
+        self.initial_states = states
+        self.states = states.copy()  # explicity do this to copy RNGSeq
         self.run_eagerly = run_eagerly
         self.history = {}
+        self.init_stage = init_stage
 
         self._jit_functions()
 
     def _jit_functions(self):
-        self.call_pred_step_jit = hooks.jit(
+        self.call_summary_step_jit = jax.jit(
+            self.call_summary_step,
+            static_argnums=[2, 3],
+        )
+        self.call_pred_step_jit = jax.jit(
             self.call_pred_step,
-            static_argnums=[1, 3],
+            static_argnums=[2, 3],
         )
-        self.call_test_step_jit = hooks.jit(
+        self.call_test_step_jit = jax.jit(
             self.call_test_step,
-            static_argnums=[2, 6],
+            static_argnums=[5, 6],
         )
-        self.call_train_step_jit = hooks.jit(
+        self.call_train_step_jit = jax.jit(
             self.call_train_step,
-            static_argnums=[2, 6],
+            static_argnums=[5, 6],
         )
 
     def __setstate__(self, d):
@@ -110,6 +103,7 @@ class ModelCore:
         del d["initial_states"]
 
         # remove jitted functions
+        del d["call_summary_step_jit"]
         del d["call_pred_step_jit"]
         del d["call_test_step_jit"]
         del d["call_train_step_jit"]
@@ -120,13 +114,38 @@ class ModelCore:
     # Abstract API
     # ----------------------------------------------------------------
 
+    def update_modules(self):
+        pass
+
+    def base_states(self) -> types.States:
+        return types.States()
+
+    def summary_step(
+        self,
+        x: tp.Any,
+        states: types.States,
+        initializing: bool,
+        training: bool,
+    ) -> tp.List[types.SummaryTableEntry]:
+        raise NotImplementedError()
+
+    def call_summary_step(
+        self,
+        x: tp.Any,
+        states: types.States,
+        initializing: bool,
+        training: bool,
+    ) -> tp.List[types.SummaryTableEntry]:
+        return utils.inject_dependencies(self.summary_step)(
+            x=x,
+            states=states,
+            initializing=initializing,
+            training=training,
+        )
+
     def pred_step(
         self,
         x: tp.Any,
-        mode: types.Mode,
-        net_params: tp.Any,
-        net_states: tp.Any,
-        rng: tp.Any,
         states: types.States,
         initializing: bool,
         training: bool,
@@ -136,40 +155,24 @@ class ModelCore:
     def call_pred_step(
         self,
         x: tp.Any,
-        mode: types.Mode,
         states: types.States,
         initializing: bool,
+        training: bool,
     ) -> PredStep:
-        get_losses_and_metrics = mode in (types.Mode.test, types.Mode.train)
-        get_summaries = mode == types.Mode.summary
 
-        with hooks.context(
-            losses=get_losses_and_metrics,
-            metrics=get_losses_and_metrics,
-            summaries=get_summaries,
-        ):
-            return utils.inject_dependencies(self.pred_step)(
-                x=x,
-                mode=mode,
-                net_params=states.net_params,
-                net_states=states.net_states,
-                rng=states.rng,
-                states=states,
-                initializing=initializing,
-                training=(mode == types.Mode.train),
-            )
+        return utils.inject_dependencies(self.pred_step)(
+            x=x,
+            states=states,
+            initializing=initializing,
+            training=training,
+        )
 
     def test_step(
         self,
         x: tp.Any,
         y_true: tp.Any,
-        mode: types.Mode,
-        net_params: tp.Any,
-        net_states: tp.Any,
-        metrics_states: tp.Any,
         sample_weight: tp.Optional[np.ndarray],
         class_weight: tp.Optional[np.ndarray],
-        rng: tp.Any,
         states: types.States,
         initializing: bool,
         training: bool,
@@ -180,38 +183,28 @@ class ModelCore:
         self,
         x: tp.Any,
         y_true: tp.Any,
-        mode: types.Mode,
         sample_weight: tp.Optional[np.ndarray],
         class_weight: tp.Optional[np.ndarray],
         states: types.States,
         initializing: bool,
+        training: bool,
     ) -> TestStep:
         return utils.inject_dependencies(self.test_step)(
-            net_params=states.net_params,
             x=x,
             y_true=y_true,
-            mode=mode,
-            net_states=states.net_states,
-            metrics_states=states.metrics_states,
             sample_weight=sample_weight,
             class_weight=class_weight,
-            rng=states.rng,
             states=states,
             initializing=initializing,
-            training=(mode == types.Mode.train),
+            training=training,
         )
 
     def grad_step(
         self,
         x: tp.Any,
         y_true: tp.Any,
-        mode: types.Mode,
-        net_params: tp.Any,
-        net_states: tp.Any,
-        metrics_states: tp.Any,
         sample_weight: tp.Optional[np.ndarray],
         class_weight: tp.Optional[np.ndarray],
-        rng: tp.Any,
         states: types.States,
         initializing: bool,
         training: bool,
@@ -222,39 +215,28 @@ class ModelCore:
         self,
         x: tp.Any,
         y_true: tp.Any,
-        mode: types.Mode,
         sample_weight: tp.Optional[np.ndarray],
         class_weight: tp.Optional[np.ndarray],
         states: types.States,
         initializing: bool,
+        training: bool,
     ) -> GradStep:
         return utils.inject_dependencies(self.grad_step)(
             x=x,
             y_true=y_true,
-            mode=mode,
-            net_params=states.net_params,
-            net_states=states.net_states,
-            metrics_states=states.metrics_states,
             sample_weight=sample_weight,
             class_weight=class_weight,
-            rng=states.rng,
             states=states,
             initializing=initializing,
-            training=(mode == types.Mode.train),
+            training=training,
         )
 
     def train_step(
         self,
         x: tp.Any,
         y_true: tp.Any,
-        mode: types.Mode,
-        net_params: tp.Any,
-        net_states: tp.Any,
-        metrics_states: tp.Any,
-        optimizer_states: tp.Any,
         sample_weight: tp.Optional[np.ndarray],
         class_weight: tp.Optional[np.ndarray],
-        rng: tp.Any,
         states: types.States,
         initializing: bool,
         training: bool,
@@ -265,30 +247,24 @@ class ModelCore:
         self,
         x: tp.Any,
         y_true: tp.Any,
-        mode: types.Mode,
         sample_weight: tp.Optional[np.ndarray],
         class_weight: tp.Optional[np.ndarray],
         states: types.States,
         initializing: bool,
+        training: bool,
     ) -> TrainStep:
         return utils.inject_dependencies(self.train_step)(
             x=x,
             y_true=y_true,
-            mode=mode,
-            net_params=states.net_params,
-            net_states=states.net_states,
-            metrics_states=states.metrics_states,
-            optimizer_states=states.optimizer_states,
             sample_weight=sample_weight,
             class_weight=class_weight,
-            rng=states.rng,
             states=states,
             initializing=initializing,
-            training=(mode == types.Mode.train),
+            training=training,
         )
 
     # ----------------------------------------------------------------
-    # *_on_batch methods
+    # high-level methods
     # ----------------------------------------------------------------
 
     def predict_on_batch(
@@ -311,14 +287,19 @@ class ModelCore:
         """
         mode = types.Mode.pred
         initializing = False
+        training = False
 
         self.maybe_initialize(mode=mode, x=x)
 
         method = self.call_pred_step if self.run_eagerly else self.call_pred_step_jit
+        states = self.states.copy() if self.run_eagerly else self.states
 
-        y_pred, state_updates, _, _, _ = method(x, mode, self.states, initializing)
-
-        self.states = self.states.merge(state_updates)
+        y_pred, self.states = method(
+            x,
+            states,
+            initializing,
+            training,
+        )
 
         return y_pred
 
@@ -355,6 +336,7 @@ class ModelCore:
         """
         mode = types.Mode.test
         initializing = False
+        training = False
 
         self.maybe_initialize(
             mode,
@@ -365,19 +347,18 @@ class ModelCore:
         )
 
         method = self.call_test_step if self.run_eagerly else self.call_test_step_jit
+        states = self.states.copy() if self.run_eagerly else self.states
 
-        loss, logs, state_updates = method(
+        loss, logs, self.states = method(
             x,
             y,
-            mode,
             sample_weight,
             class_weight,
-            self.states,
+            states,
             initializing,
+            training,
         )
 
-        self.states = self.states.merge(state_updates)
-
         return logs
 
     def train_on_batch(
@@ -419,6 +400,7 @@ class ModelCore:
         """
         mode = types.Mode.train
         initializing = False
+        training = True
 
         self.maybe_initialize(
             mode=mode,
@@ -429,115 +411,202 @@ class ModelCore:
         )
 
         method = self.call_train_step if self.run_eagerly else self.call_train_step_jit
+        states = self.states.copy() if self.run_eagerly else self.states
 
-        logs, state_updates = method(
+        logs, self.states = method(
             x,
             y,
-            mode,
             sample_weight,
             class_weight,
-            self.states,
+            states,
             initializing,
+            training,
         )
 
-        self.states = self.states.merge(state_updates)
-
         return logs
 
-    # ----------------------------------------------------------------
-    # other methods
-    # ----------------------------------------------------------------
+    def summary(
+        self,
+        x,
+        depth: int = 2,
+        tablefmt: str = "fancy_grid",
+        return_repr: bool = False,
+        **tablulate_kwargs,
+    ) -> tp.Optional[str]:
+        """
+        Prints a summary of the network.
+        Arguments:
+            x: A sample of inputs to the network.
+            depth: The level number of nested level which will be showed.
+                Information about summaries from modules deeper than `depth`
+                will be aggregated together.
+            tablefmt: A string representing the style of the table generated by
+                `tabulate`. See
+                [python-tabulate](https://github.com/astanin/python-tabulate)
+                for more options.
+            tablulate_kwargs: Additional keyword arguments passed to `tabulate`.
+                See [python-tabulate](https://github.com/astanin/python-tabulate)
+                for more options.
+        """
+        mode = types.Mode.pred
+        initializing = False
+        training = False
 
-    def update_modules(self):
-        pass
+        self.maybe_initialize(mode, x=x)
 
-    def reset(self):
-        self.states = self.initial_states.copy()
+        method = (
+            self.call_summary_step if self.run_eagerly else self.call_summary_step_jit
+        )
+        states = self.states.copy() if self.run_eagerly else self.states
 
-    def reset_metrics(self):
-        self.states = self.states.update(
-            metrics_states=self.initial_states.metrics_states
+        entries = method(
+            x,
+            states,
+            initializing,
+            training,
+        )
+        total_entry = entries[-1]
+        entries = entries[:-1]
+
+        depth_groups: tp.Dict[str, tp.List[types.SummaryTableEntry]] = toolz.groupby(
+            lambda entry: "/".join(entry.path.split("/")[:depth]), entries
         )
 
-    def maybe_initialize(
-        self,
-        mode: types.Mode,
-        x: tp.Union[np.ndarray, tp.Mapping[str, tp.Any], tp.Tuple] = (),
-        y_true: tp.Union[np.ndarray, tp.Mapping[str, tp.Any], tp.Tuple, None] = None,
-        sample_weight: tp.Optional[np.ndarray] = None,
-        class_weight: tp.Optional[np.ndarray] = None,
-    ):
-        rng = self.states.rng if isinstance(self.states.rng, types.RNGSeq) else None
-        initializing = True
-        state_updates: types.States
+        def get_grouped_entry(
+            entry: types.SummaryTableEntry,
+        ) -> types.SummaryTableEntry:
+            group = depth_groups[entry.path]
 
-        if (
-            mode in (types.Mode.pred, types.Mode.summary)
-            and isinstance(self.states.net_params, types.Uninitialized)
-            and isinstance(self.states.net_states, types.Uninitialized)
-        ):
-            method = (
-                self.call_pred_step if self.run_eagerly else self.call_pred_step_jit
+            return types.SummaryTableEntry(
+                path=entry.path,
+                module_type_name=entry.module_type_name,
+                output_value=entry.output_value,
+                trainable_params_count=sum(
+                    entry_.trainable_params_count for entry_ in group
+                ),
+                trainable_params_size=sum(
+                    entry_.trainable_params_size for entry_ in group
+                ),
+                non_trainable_params_count=sum(
+                    entry_.non_trainable_params_count for entry_ in group
+                ),
+                non_trainable_params_size=sum(
+                    entry_.non_trainable_params_size for entry_ in group
+                ),
             )
 
-            _, state_updates, _, _, _ = method(
-                x,
-                mode,
-                self.states,
-                initializing,
+        entries = [
+            get_grouped_entry(entry) for entry in entries if entry.path in depth_groups
+        ]
+
+        def format_output(value) -> str:
+            file = StringIO()
+            outputs = jax.tree_map(lambda x: f"{x.shape}{{pad}}  {x.dtype}", value)
+            yaml.safe_dump(
+                outputs, file, default_flow_style=False, indent=2, explicit_end=False
             )
-        elif mode == types.Mode.test and isinstance(
-            self.states.metrics_states, types.Uninitialized
-        ):
-            method = (
-                self.call_test_step if self.run_eagerly else self.call_test_step_jit
+            return file.getvalue().replace("\n...", "")
+
+        def format_size(size):
+            return (
+                f"{size / 1e9 :,.1f} GB"
+                if size > 1e9
+                else f"{size / 1e6 :,.1f} MB"
+                if size > 1e6
+                else f"{size / 1e3 :,.1f} KB"
+                if size > 1e3
+                else f"{size:,} B"
             )
 
-            _, _, state_updates = method(
-                x,
-                y_true,
-                mode,
-                sample_weight,
-                class_weight,
-                self.states,
-                initializing,
-            )
-        elif mode == types.Mode.train and isinstance(
-            self.states.optimizer_states, types.Uninitialized
-        ):
-            method = (
-                self.call_train_step if self.run_eagerly else self.call_train_step_jit
+        table: tp.List = [
+            [
+                "Inputs",
+                format_output(x),
+                "",
+                "",
+            ]
+        ]
+
+        for entry in entries:
+
+            table.append(
+                [
+                    f"{entry.path}{{pad}}  {entry.module_type_name}",
+                    format_output(entry.output_value),
+                    f"{entry.trainable_params_count:,}{{pad}}    {format_size(entry.trainable_params_size)}"
+                    if entry.trainable_params_count > 0
+                    else "",
+                    f"{entry.non_trainable_params_count:,}{{pad}}    {format_size(entry.non_trainable_params_size)}"
+                    if entry.non_trainable_params_count > 0
+                    else "",
+                ]
             )
 
-            _, state_updates = method(
-                x,
-                y_true,
-                mode,
-                sample_weight,
-                class_weight,
-                self.states,
-                initializing,
+        # add padding
+        for col in range(4):
+            max_length = max(
+                len(line.split("{pad}")[0])
+                for row in table
+                for line in row[col].split("\n")
             )
 
-        else:
-            return
+            for row in table:
+                row[col] = "\n".join(
+                    line.format(
+                        pad=" " * (max_length - len(line.rstrip().split("{pad}")[0]))
+                    )
+                    for line in row[col].rstrip().split("\n")
+                )
 
-        if mode in (types.Mode.pred, types.Mode.test, types.Mode.train):
-            if isinstance(state_updates.net_params, types.Uninitialized):
-                state_updates = state_updates.update(net_params=None)
+        # global summaries
+        params_count = total_entry.trainable_params_count
+        params_size = total_entry.trainable_params_size
+        states_count = total_entry.non_trainable_params_count
+        states_size = total_entry.non_trainable_params_size
+        total_count = params_count + states_count
+        total_size = params_size + states_size
 
-            if isinstance(state_updates.net_states, types.Uninitialized):
-                state_updates = state_updates.update(net_states=None)
-        if mode in (types.Mode.test, types.Mode.train):
-            if isinstance(state_updates.metrics_states, types.Uninitialized):
-                state_updates = state_updates.update(metrics_states=None)
+        summary = (
+            "\n"
+            + tabulate(
+                table,
+                headers=[
+                    "Layer",
+                    "Outputs Shape",
+                    "Trainable\nParameters",
+                    "Non-trainable\nParameters",
+                ],
+                tablefmt=tablefmt,
+                **tablulate_kwargs,
+            )
+            + "\n"
+            + tabulate(
+                [
+                    [
+                        f"Total Parameters:",
+                        f"{total_count:,}",
+                        f"{format_size(total_size)}" if total_count > 0 else "",
+                    ],
+                    [
+                        f"Trainable Parameters:",
+                        f"{params_count:,}",
+                        f"{format_size(params_size)}" if params_count > 0 else "",
+                    ],
+                    [
+                        f"Non-trainable Parameters:",
+                        f"{states_count:,}",
+                        f"{format_size(states_size)}" if states_count > 0 else "",
+                    ],
+                ],
+                tablefmt="plain",
+            )
+            + "\n"
+        )
 
-        if mode == types.Mode.train:
-            if isinstance(state_updates.optimizer_states, types.Uninitialized):
-                state_updates = state_updates.update(optimizer_states=None)
+        print(summary)
 
-        self.states = self.states.merge_new(state_updates)
-        self.initial_states = self.initial_states.merge_new(state_updates)
+        if return_repr:
+            return summary
 
     def save(
         self,
@@ -613,3 +682,81 @@ class ModelCore:
         self.initial_states = cloudpickle.loads(
             (path / "initial_states.pkl").read_bytes()
         )
+
+    def reset(self):
+        self.states = self.initial_states.copy()
+
+    def reset_metrics(self):
+        if hasattr(self.initial_states, "metrics_states"):
+            self.states = self.states.update(
+                metrics_states=self.initial_states.metrics_states
+            )
+
+    # ----------------------------------------------------------------
+    # other methods
+    # ----------------------------------------------------------------
+
+    def maybe_initialize(
+        self,
+        mode: types.Mode,
+        x: tp.Union[np.ndarray, tp.Mapping[str, tp.Any], tp.Tuple] = (),
+        y_true: tp.Union[np.ndarray, tp.Mapping[str, tp.Any], tp.Tuple, None] = None,
+        sample_weight: tp.Optional[np.ndarray] = None,
+        class_weight: tp.Optional[np.ndarray] = None,
+    ):
+
+        if mode <= self.init_stage:
+            return
+
+        initializing = True
+        training = True
+        state_updates: types.States
+
+        if mode == types.Mode.pred:
+            method = (
+                self.call_pred_step if self.run_eagerly else self.call_pred_step_jit
+            )
+            states = self.states.copy() if self.run_eagerly else self.states
+
+            _, state_updates = method(
+                x,
+                states,
+                initializing,
+                training,
+            )
+        elif mode == types.Mode.test:
+            method = (
+                self.call_test_step if self.run_eagerly else self.call_test_step_jit
+            )
+            states = self.states.copy() if self.run_eagerly else self.states
+
+            _, _, state_updates = method(
+                x,
+                y_true,
+                sample_weight,
+                class_weight,
+                states,
+                initializing,
+                training,
+            )
+        elif mode == types.Mode.train:
+            method = (
+                self.call_train_step if self.run_eagerly else self.call_train_step_jit
+            )
+            states = self.states.copy() if self.run_eagerly else self.states
+
+            _, state_updates = method(
+                x,
+                y_true,
+                sample_weight,
+                class_weight,
+                states,
+                initializing,
+                training,
+            )
+        else:
+            raise ValueError(f"Invalid mode '{mode}'")
+
+        self.init_stage = mode
+        self.states = self.states.maybe_update(**state_updates)
+        self.initial_states = self.initial_states.maybe_update(**state_updates)
