@@ -1,22 +1,23 @@
-import haiku as hk
+from elegy import utils
+
 import jax.numpy as jnp
-from haiku.testing import transform_and_run
 from unittest import TestCase
 
 import elegy
 
 
 class DropoutTest(TestCase):
-    @transform_and_run
     def test_dropout_connects(self):
-        elegy.nn.Dropout(0.25)(jnp.ones([3, 3]), is_training=True)
+        elegy.nn.Dropout(0.25).call_with_defaults(rng=elegy.RNGSeq(42))(
+            jnp.ones([3, 3]), training=True
+        )
 
     def test_on_predict(self):
         class TestModule(elegy.Module):
-            def call(self, x, is_training):
-                return elegy.nn.Dropout(0.5)(x, is_training)
+            def call(self, x, training):
+                return elegy.nn.Dropout(0.5)(x, training)
 
-        model = elegy.Model(module=TestModule.defer())
+        model = elegy.Model(TestModule())
 
         x = jnp.ones([3, 5])
 
